/*
	File:		TLinkedListExample.cp

	Contains:	This module shows an example use of the TLinkedList class.

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

#include "TInitSLM.h"			// the TInitSLM class and SLM include files
#include "TListExample.h"

///
///	CONSTANTS
///

const unsigned	kMaxPoints = 20;		// maximum number of points
const unsigned	kNumPoints = 5;			// number of points for now

///
///	PROTOTYPES
///

static void build_list_of_points( TLinkedList*	thePointList );
static void remove_list_of_points( TLinkedList*	thePointList );
static void disp_list_of_points( TIterator*	iterator );


/*
	main 
	
	This examples creates list of TPoints using the TLinkedList. The TLinkedList has 
	more versatility as far as manipulating the list then a TSimpleList because of the
	addition of the AddAfter, AddBefore, AddLinkAfter, and AddLinkBefore methods. In
	this example we are not creating a TLink seperately, so we need to uses AddFirst,
	and AddAfter calls which requires only a pointer to an object; otherwise, we
	could've created our own TLink and used AddLinkFirst or AddLinkAfter. 
	Once the list has been created it uses an iterator to index thru the list and
	display the content of thes list. Once displayed it deletes the list and its items.
*/

main() 
{	
	TInitSLM initLibraryManager;			// initialize the shared library manager
	
	if( initLibraryManager.Failed() )	// If we failed, let go home
		return 1;

	// Allocate the pool for TLinkedList's TLinks. Since the size of the blocks
	// is constant, it's better to use a TChunkyPool.
	size_t poolsize = kMaxPoints * (sizeof(TLink) + kChunkyPoolChunkOverhead);
	TChunkyPool	*listpool = new (poolsize,kApplicZone) TChunkyPool(sizeof(TLink));
	
	TLinkedList	*thepointlist = new TLinkedList(listpool);
	
	if( thepointlist ) {				// This is sample of a linked list of points.
		
		build_list_of_points( thepointlist );// create a list of TPoints
		
		if( !thepointlist->IsEmpty() ) { // make sure list is not empty
			OSErr	err;
			TPoint*	thePt;
	
			cout << "Total points = " << thepointlist->Count() << endl;
			
			// Try adding a point that we know already exists. We'll use a match object
			// because we want it to do the comparing of points rather than just
			// letting the list compare pointers to TPoints.
			cout << "Adding point (2,20)" << endl; 
			thePt = new TPoint(2,20);
			err = thepointlist->AddUnique(thePt,TPointMatchObject(thePt));
			if (err == kNoError)
				cout << "Hey, that point should already have been in the list!" << endl;
			else
			{
				delete thePt;
				cout << "Adding point (2,20) failed as expected." << endl;
			}
			
			// Now try adding a point that we know isn't in the list
			cout << "Adding point (2,22)" << endl; 
			thePt = new TPoint(2,22);
			err = thepointlist->AddUnique(thePt,TPointMatchObject(thePt));
			if (err != kNoError)
			{
				delete thePt;
				cout << "Hey, that point shouldn't have already have been in the list!" << endl;
			}
			cout << "Adding point (2,22) succeeded as expected." << endl; 
			
			TRY						// use an iterator to display our points
				TIterator *iterate = thepointlist->CreateIterator(nil);
				FailNULL( iterate, kNoError, "unable to CreateIterator");
				disp_list_of_points( iterate );// display the points
			CATCH_ALL
				cout <<"Error: Sorry unable to create an iterator" << endl;
			ENDTRY
			remove_list_of_points(thepointlist);// now we are done with list 
		}
		else
			cout << "Empty lists\n" << endl;
		delete thepointlist;			// we are done with simplelist
		delete listpool;				// free the memory
	}
	
	return 0;
}

/*
	build_list_of_points
	
	Create a list of points
*/

static void build_list_of_points( TLinkedList*	thePointList )
{
	cout << "Building list of points" << endl;
	TPoint		*prevtpoint = nil;
	for( int i=0; i< kNumPoints; i++ ) {
		TPoint	*tpoint = new TPoint(i,i*10); // create a point
		
		if( i==0 )							// if the first item add it first
			thePointList->AddFirst( tpoint );
		else								
			thePointList->AddAfter( tpoint, prevtpoint );
		prevtpoint = tpoint;
	}
}

/*
	remove_list_of_points
	
	Remove and delete all elements from the list of points
*/

static void remove_list_of_points( TLinkedList*	thePointList )
{
	// Call DeleteAll to remove and delete all the objects
	thePointList->DeleteAll();
}

/*
	disp_list_of_points
	
	iterator thru the list and display the values of the points
*/

static void disp_list_of_points( TIterator*	iterator )
{
	iterator->Reset();					// reset the iterator to begining of the list
	TPoint 	*thept;
	int count=0;
	while( thept = (TPoint *)iterator->Next() ) {
		Point pt = thept->GetPoint();
		cout << "point[" << count++ << "] = (" << (int)pt.h << ", " << (int)pt.v << ")" <<endl;
	}
}
