/*
	File:		TPriorityListExample.cp

	Contains:	MPW tool for demonstrating how to use the TPriorityList class.

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

#include "TInitSLM.h"			// the TInitSLM class and SLM include files
#include "TPriorityListExample.h"

#include <stdlib.h>		// for rand and srand
#include <Events.h>		// for TickCount

///
///	CONSTANTS
///

const unsigned kTotalTasks = 5;				// number of tasks

///
///	PROTOTYPES
///

static void build_priority_task( TPriorityList* priorityList );
static void iterate_tasks(TIterator* iterator );
static void remove_tasks( TPriorityList* priorityList );
static void TaskA();
static void TaskB();
static void TaskC();
static void TaskD();
static void TaskE();

///
///	GLOBALS
///

// gTask is an array of pointers to routines that will do a simple task
TaskProcPtr gTasks[kTotalTasks] = {TaskA, TaskB, TaskC, TaskD, TaskE};

/*
	main 
	
	This examples creates list of TTasks using the TPriorityList. Since it not creating
	a TLink seperately, it creates its own pool, and uses the AddPrioritized call, which 
	requires only a pointer to an object and a priority. Otherwise we could've created
	our own TPriorityLink and used AddLink. Since each item added to the list has a 
	priority, when we iteratate over the list they items will be returned in order
	of their priority.  Once the list has been created it uses an
	iterator to index thru the list and call the task proc of each TTask in the list.
	Once this is done it deletes the list and its items.
*/

main() 
{	
	TInitSLM initLibraryManager;				// initialize the shared library manager
	
	if( initLibraryManager.Failed() )		// If we failed, let's go home
		return 1;
	
	// Allocate the pool for TPriorityList's TLinks. Since the size of the blocks
	// is constant, it's better to use a TChunkyPool.
	size_t poolsize = kTotalTasks * (sizeof(TPriorityLink) + kChunkyPoolChunkOverhead);
	TChunkyPool *listpool = new (poolsize,kApplicZone) TChunkyPool(sizeof(TPriorityLink));
	
	TPriorityList 	*tasklist = new TPriorityList(listpool);
											
	if( tasklist ) {					
		build_priority_task( tasklist );	// build a list of prioritized tasks

		if( ! tasklist->IsEmpty() ) {		// make sure list is not empty
			
			cout << "Total no. of tasks = " << tasklist->Count() << endl;
											
			TRY							// let's go through the list of using our TInterator
				TIterator *iter = tasklist->CreateIterator(nil);
				FailNULL( iter, noErr, "unable to CreateIterator");
				iterate_tasks( iter );
				delete iter;
			CATCH_ALL
				cout <<"Error: Sorry unable to create an iterator" << endl;
			ENDTRY
			remove_tasks( tasklist );		// now we are done so let's remove the tasks
		}
		else
			cout << "Empty lists\n" << endl;
		delete tasklist;					// we are done with list
		delete listpool;					// free the pool
	}
	return 0;
}

///
///	build_priority_task
///
///	Build a list of prioritized tasks
///

static void build_priority_task( TPriorityList* priorityList )
{
	cout << "\nBuilding a list of tasks with priority.\n" << endl;

	srand(int(TickCount()));		// start with a random seed
	for( int i=0; i< kTotalTasks; i++ ) {
		// create a TTask and initialize it with one of the task procs.
		TTask		*ttask = new TTask(gTasks[i]); 	
		
		// add the task object to the list with a random priority
		priorityList->AddPrioritized( ttask, rand() );
	}
}

///
///	iterate_tasks
///
///	Iterator thru the list of tasks and execute them in order of their priority
///

static void iterate_tasks(TIterator* iterator )
{
	int i=0;
	TTask 	*thetask;
	
	iterator->Reset();				// make iterator point to begining of the list
	while( thetask = (TTask *)iterator->Next() ) {
		cout << "iterating task[" << i++ << "]";
		(*thetask->GetTask())();	// call the TTask's task proc
		cout << endl;
	}
}

///
///	remove_tasks
///
///	Remove each TTask from the list and then delete it. We could do the same
/// thing by calling DeleteAll.
///

static void remove_tasks( TPriorityList* priorityList )
{
	TTask 	*thetask;
	while( thetask = (TTask *)priorityList->RemoveFirst())
		delete thetask;
}

//
//	TaskA
//
static void TaskA()
{
	cout << " Task A executing" << endl;
}

//
//	TaskB
//
static void TaskB()
{
	cout << " Task B executing" << endl;
}

//
//	TaskC
//
static void TaskC()
{
	cout << " Task C executing" << endl;
}

//
//	TaskD
//
static void TaskD()
{
	cout << " Task D executing" << endl;
}

//
//	TaskE
//
static void TaskE()
{
	cout << " Task E executing" << endl;
}
